<?php
if (!defined('ABSPATH')) {
  exit;
}

class CustoMeow_Admin_Dashboard {

  public static $_instance;

  /**
   * @return CustoMeow_Admin_Dashboard
   */
  public static function instance() {

    if (is_null(self::$_instance)) {
      self::$_instance = new self();
    }

    return self::$_instance;
  }

  function __construct() {
    add_action('admin_enqueue_scripts', array($this, 'add_admin_scripts'));
    add_action('admin_enqueue_scripts', array($this, 'add_admin_load_text_domain'), 100);
  }

  /**
   * 注册 js 文件
   * @param {*} $hook
   * @return {*}
   */
  public static function add_admin_scripts($hook) {
    if (strpos($hook, 'customeow-dashboard') !== false) {
      $script_path = '/build/admin-bootstrap.js';
      $script_asset_path = dirname(CUSTOMEOW_PP_PLUGIN_FILE) . '/build/admin-bootstrap.asset.php';
      $script_asset = file_exists($script_asset_path)
      ? require $script_asset_path
      : array(
        'dependencies' => array(),
        'version' => filemtime($script_path),
      );
      $script_url = plugins_url($script_path, CUSTOMEOW_PP_PLUGIN_FILE);

      wp_register_script(
        'customeow_pp_admin_script',
        $script_url,
        $script_asset['dependencies'],
        $script_asset['version'],
        true
      );

      wp_register_style(
        'customeow_pp_admin_style',
        plugins_url('/build/admin-bootstrap.css', CUSTOMEOW_PP_PLUGIN_FILE),
        array('wp-components'),
        filemtime(dirname(CUSTOMEOW_PP_PLUGIN_FILE) . '/build/admin-bootstrap.css')
      );

      wp_enqueue_script('customeow_pp_admin_script');
      wp_enqueue_style('customeow_pp_admin_style');

      CustoMeow_Admin_Dashboard::render_connect();
    }
  }

  /**
   * 注册前端翻译文件.
   */
  public static function add_admin_load_text_domain() {
    $path = dirname(CUSTOMEOW_PP_PLUGIN_FILE) . '/languages';

    wp_set_script_translations('customeow_pp_admin_script', 'customeow-product-personalizer', $path);
  }

  /**
   * @description: 检查是否开启了简单 URL
   * @return {*}
   */
  private static function check_permalinks() {
    $permalinks = get_option('permalink_structure', false);
    if ($permalinks && strlen($permalinks) > 0) {
      return true;
    }
    return false;
  }

  /**
   * @description: 收集数据渲染页面
   * @return {*}
   */
  public static function render_connect() {

    $issues = array();

    // 是否设置了固定链接
    $permalinks_set = self::check_permalinks();

    if (!$permalinks_set) {
      /* translators: %s WP Options Permalink URL link. */
      $message = __('WooCommerce API will not work unless your permalinks are set up correctly. Go to <a href="%s">Permalinks settings</a> and make sure that they are NOT set to "plain".', 'customeow-product-personalizer');
      $settings_url = admin_url('options-permalink.php');
      $issues[] = sprintf($message, $settings_url);
    }

    // 不允许本地安装使用
    if (strpos(get_site_url(), 'localhost')) {
      $issues[] = __('You can\'t connect to CustoMeow from localhost. CustoMeow needs to be able reach your site to establish a connection.', 'customeow-product-personalizer');
    };

    $connect_status = CustoMeow_Integration::instance()->is_connected();

    // 准备连接的数据
    $connects = array(
      'connect_url' => CustoMeow_Admin_Dashboard::get_connect_url(),
      'home_url' => get_home_url(),
      'name' => get_bloginfo('title', 'display'),
      'issues' => $issues,
      'ajax_url' => esc_url(admin_url('admin-ajax.php')),
      "security" => wp_create_nonce('check_customeow_connect_status'),
      'connect_status' => (int) $connect_status,
    );
    $connects['store_url'] = CustoMeow_Integration::instance()->get_option("store_url");
    $connects['shop_config'] = CustoMeow_Integration::instance()->get_option("shop_config");
    wp_localize_script('customeow_pp_admin_script', 'customeow_admin_data', $connects);
  }

  public static function get_connect_url() {
    return CustoMeow_Utilities::get_customeow_host() . 'store';
  }

}