<?php
if (!defined('ABSPATH')) {
  exit;
}

class CustoMeow_Client {
  private $key = false;
  private $userAgent = 'CustoMeow WooCommerce Plugin';
  private $apiUrl;

  public function __construct($apiUrl, $key) {
    $this->key = $key;
    $this->userAgent .= ' ' . CustoMeow_Utilities::VERSION . ' (WP ' . get_bloginfo('version') . ' + WC ' . WC()->version . ')';

    if (!function_exists('json_decode') || !function_exists('json_encode')) {
      throw new CustoMeowException('Requires the PHP JSON extension to work!');
    }

    if ($apiUrl) {
      $this->apiUrl = $apiUrl;
    } else {
      $this->apiUrl = CustoMeow_Utilities::get_customeow_go_host();
    }
  }

  public function get($path, $params = array()) {
    return $this->request('GET', $path, $params);
  }

  public function post($path, $data = array(), $params = array()) {
    return $this->request('POST', $path, $params, $data);
  }

  private function request($method, $path, array $params = array(), $data = null) {
    $url = trim($path, '/');

    if (!empty($params)) {
      $url .= '?' . http_build_query($params);
    }

    $authHeader = array();

    if (!empty($key)) {
      $authHeader = array('CustoMeow-Api-TOKEN' => $this->key);
    }
    $request = array(
      'timeout' => 10,
      'user-agent' => $this->userAgent,
      'method' => $method,
      'headers' => $authHeader,
      'body' => $data !== null ? wp_json_encode($data) : null,
    );
// var_dump($request);
    if ($method === 'POST') {
      $result = wp_remote_post($this->apiUrl . $url, $request);

      $result = apply_filters('customeow_api_result', $result, $method, $this->apiUrl . $url, $request);

      if (is_wp_error($result)) {
        throw new CustoMeowException('API request failed - ' . esc_html($result->get_error_message()));
      }

      $body = wp_remote_retrieve_body($result);

      $response = json_decode($body, true);

      return $response;
    } else if ($method === 'GET') {
      $result = wp_remote_get($this->apiUrl . $url, $request);

      $result = apply_filters('customeow_api_result', $result, $method, $this->apiUrl . $url, $request);

      if (is_wp_error($result)) {
        throw new CustoMeowException('API request failed - ' . esc_html($result->get_error_message()));
      }

      $body = wp_remote_retrieve_body($result);

      $response = json_decode($body, true);

      return $response;
    }

  }
}

/**
 * Class CustoMeowException Generic CustoMeow exception
 */
class CustoMeowException extends Exception {}
