<?php
if (!defined('ABSPATH')) {
  exit;
}

class CustoMeow_Integration {
  /** 缓存连接状态key */
  const CUSTOMEOW_API_CONNECT_STATUS = 'customeow_api_connect_status';
  /** 存储配置 key */
  public static $customeow_settings_key = 'woocommerce_customeow_settings';

  public static $_instance;

  public static function instance() {
    if (is_null(self::$_instance)) {
      self::$_instance = new self();
    }

    return self::$_instance;
  }

  public function __construct() {
    self::$_instance = $this;
  }

  public function get_client($api_url = null, $key = null) {
    require_once 'class-customeow-client.php';

    $client = new CustoMeow_Client($api_url, $key);

    return $client;
  }

  public function is_connected($force = false) {

    // $options = get_option('woocommerce_customeow_settings', array());
    // if (empty($options)) {
    //   return false;
    // }

    // 取缓存结果 TODO: 删除店铺需要后端主动请求插件设置状态
    // if (!$force) {
    //   $connected = get_transient(self::CUSTOMEOW_API_CONNECT_STATUS);
    //   if ($connected && $connected['status'] == 1) {
    //     return true;
    //   } else if ($connected && $connected['status'] == 0) {
    //     return false;
    //   }
    // }

    $go_url = CustoMeow_Utilities::get_customeow_go_host();
    $client = $this->get_client($go_url);

    $response = false;

    // 根据店铺获取连接状态并做缓存
    try {
      $storeData = $client->post('/api/v1/store-platform/user-store-info', array(
        'storeURL' => get_home_url(),
      ));


      if (!empty($storeData) && $storeData['code'] === 0 && $storeData['data']) {
        $response = $storeData['data']['isConnect'];

        $options = $this->get_settings(array());
        $options['is_connect'] = $storeData['data']['isConnect'];

        if (isset($storeData['data']['storeURL'])) {
          // 定制喵店铺 url
          $options['store_url'] = $storeData['data']['storeURL'];
        }

        $options['shop_config'] = $storeData['data']['storeConfig'];
        // 店铺 id 
        $options['store_id'] = $storeData['data']['id'];
        // 平台 id 用于C端提交定制数据
        $options['platform_id'] = $storeData['data']['platformStoreId'];

        $this->update_settings($options);

        if ($response && $storeData['data']['isConnect']) {
          set_transient(self::CUSTOMEOW_API_CONNECT_STATUS, array('status' => 1));
        } else {
          set_transient(self::CUSTOMEOW_API_CONNECT_STATUS, array('status' => 0), MINUTE_IN_SECONDS);
        }
      }
    } catch (Exception $e) {
      $response = false;
      set_transient(self::CUSTOMEOW_API_CONNECT_STATUS, array('status' => 0), MINUTE_IN_SECONDS);
    }

    return $response;
  }

  /**
   * @description: 前端发起请求调用外部服务更新店铺数据
   * @return {*}
   */
  public static function ajax_force_check_customeow_connect_status() {
    CustoMeow_Admin::validateAdminAccess();

    check_admin_referer('check_customeow_connect_status', 'security');

    // 检查连接状态并获取配置
    if (self::instance()->is_connected(true)) {

      $options = get_option(self::$customeow_settings_key, array());
      wp_send_json_success($options);
      wp_die();
    } else {
      wp_send_json_error(array('is_connect' => false));
      wp_die();
    }
  }

  /**
   * 获取指定 name 的配置
   * @param {*} $name
   * @param {*} $default
   * @return {*}
   */
  public function get_option($name, $default = array()) {
    $options = $this->get_settings($default);
    if (!empty($options[$name])) {
      return $options[$name];
    }

    return false;
  }

  /**
   *  获取 woocommerce_customeow_settings 配置
   * @param {*} $default
   * @return {*}
   */
  public function get_settings($default = array()) {
    return get_option(self::$customeow_settings_key, $default);
  }

  /**
   * 保存 woocommerce_customeow_settings 配置
   * @param {*} $settings
   * @return {*}
   */
  public function update_settings($settings) {
    update_option(self::$customeow_settings_key, $settings);
  }
}