<?php
if (!defined('ABSPATH')) {
  exit;
}

class CustoMeow_Request_Log {
  const CUSTOMEOW_USER_AGENT = 'WooCommerce API Client-Golang/1.0.3';
  const CUSTOMEOW_INCOMING_API_ERRORS = 'customeow-incoming-api-error';
  const CUSTOMEOW_OUTGOING_API_ERRORS = 'customeow-outgoing-api-error';
  const CUSTOMEOW_OUTGOING_API_ERRORS_DEV = 'customeow-outgoing-api-dev';

  private static $instance = null;

  public static function init() {
    if (self::$instance === null) {
      self::$instance = new self();
    }

    add_filter('woocommerce_api_serve_request', array(self::$instance, 'log_incoming_customeow_api_requests'), 10, 3);
    add_filter('customeow_api_result', array(self::$instance, 'log_outgoing_customeow_api_requests'), 10, 4);
  }

  public function log_incoming_customeow_api_requests($served, $result, $request) {

    // 不是请求 CustoMeow Product Personalizer Reset API 的请求不理会
    if (!$this->isCustoMeowPDApiRequest($request)) {
      return $served;
    }

    if (!empty($result['errors'])) {
      $this->save_to_wc_log($request, $result, self::CUSTOMEOW_INCOMING_API_ERRORS);
    }

    return $served;
  }

  public function log_outgoing_customeow_api_requests($result, $method, $url, $request) {

    // 方便开发环境查看接口数据
    if (CustoMeow_Utilities::ENV != 'prod') {
      $this->save_to_wc_log($request, $result, self::CUSTOMEOW_OUTGOING_API_ERRORS_DEV);
    }

    $status = wp_remote_retrieve_response_code($result);
    $code_success = ($status < 200 || $status >= 300);

    // 非开发环境下只收集错误日志
    if (is_wp_error($result) || $code_success) {
      $this->save_to_wc_log($request, $result, self::CUSTOMEOW_OUTGOING_API_ERRORS);
    }

    return $result;
  }

  private function save_to_wc_log($request, $result, $context) {
    if (!function_exists('wc_get_logger')) {
      return false;
    }

    $logger = wc_get_logger();
    $context = array('source' => $context);
    $log_item = array(
      'request' => (array) $request,
      'results' => (array) $result,
    );
    $logger->error(wc_print_r($log_item, true), $context);

    return true;
  }

  /**
   * @description: 是否请求 CustoMeow PP 插件 暴露的 api
   * @param {*} $request
   * @return {*}
   */
  private function isCustoMeowPDApiRequest($request) {
    if (!empty($request->headers) && !empty($request->headers['USER_AGENT']) && $request->headers['USER_AGENT'] == self::CUSTOMEOW_USER_AGENT) {
      return true;
    }

    return false;
  }
}