<?php

class CustoMeow_PP_REST_API_Controller extends WC_REST_Controller {
  protected $namespace = 'wc/v3';
  protected $rest_base = 'customeow';

  public function register_routes() {
    register_rest_route($this->namespace, '/' . $this->rest_base . '/store', array(
      array(
        'methods' => WP_REST_Server::READABLE,
        'permission_callback' => array($this, 'get_items_permissions_check'),
        'callback' => array($this, 'get_store_data'),
        'show_in_index' => true,
      ),
    ));

    register_rest_route($this->namespace, '/' . $this->rest_base . '/store-config', array(
      array(
        'methods' => WP_REST_Server::EDITABLE,
        'callback' => array($this, 'set_customeow_store'),
        'permission_callback' => array($this, 'get_items_permissions_check'),
        'show_in_index' => false,
        'args' => array(
          'id' => array(
            'required' => false,
            'type' => 'string',
            'description' => 'Store Id',
          ),
          'isConnect' => array(
            'required' => true,
            'type' => 'boolean',
            'description' => 'Store Connection Status',
          ),
          'storeURL' => array(
            'required' => false,
            'type' => 'string',
            'description' => 'Store Url',
          ),
          'platformStoreId' => array(
            'required' => false,
            'type' => 'string',
            'description' => 'Platform Id',
          ),
          'storeConfig' => array(
            'required' => false,
            'type' => 'object',
            'description' => 'Shop Config',
          ),
        ),
      ),
    ));
  }

  public static function get_store_data() {
    $theme = wp_get_theme();

    return array(
      'website' => get_site_url(),
      'version' => WC()->version,
      'name' => get_bloginfo('title', 'display'),
      'theme' => array(
        'name' => $theme->get('Name'),
        'varsion' => $theme->get('Version'),
        'author' => $theme->get('Author'),
        'description' => $theme->get('Description'),
        'textDomain' => $theme->get('TextDomain'),
        'template' => $theme->get('Template'),
        'themeURI' => $theme->get('ThemeURI'),
        'authorURI' => $theme->get('AuthorURI'),
      ),
    );
  }

  public static function set_customeow_store($request) {
    $error = false;

    $options = get_option('woocommerce_customeow_settings', array());

    $store_id = $request->get_param('id');
    $platform_id = $request->get_param('platformStoreId');
    $is_connect = $request->get_param('isConnect');
    $store_url = $request->get_param('storeURL');
    $shop_config = $request->get_param('storeConfig');

    $options['store_id'] = $store_id;
    $options['platform_id'] = $platform_id;
    $options['is_connect'] = $is_connect;
    $options['store_url'] = $store_url;
    $options['shop_config'] = $shop_config;

    if ($is_connect) {
      set_transient(CustoMeow_Integration::CUSTOMEOW_API_CONNECT_STATUS, array('status' => 1));
    } else {
      set_transient(CustoMeow_Integration::CUSTOMEOW_API_CONNECT_STATUS, array('status' => 0), MINUTE_IN_SECONDS);
    }

    CustoMeow_Integration::instance()->update_settings($options);

    return array(
      'error' => $error,
      'data' => $options,
    );
  }

  /**
   * 检查给定请求是否拥有读取打印端点的权限。
   */
  public function get_items_permissions_check($request) {
    if (!wc_rest_check_user_permissions('read')) {
      return new WP_Error('woocommerce_rest_cannot_view', __('Sorry, you cannot list resources.', 'woocommerce'), array('status' => rest_authorization_required_code()));
    }

    return true;
  }

  /**
   * 检查给定请求是否有更新产品的权限。
   */
  public function update_item_permissions_check($request) {
    $params = $request->get_url_params();
    $product = wc_get_product((int) $params['product_id']);

    if (empty($product) && !wc_rest_check_post_permissions('product', 'edit', $product->get_id())) {
      return new WP_Error('woocommerce_rest_cannot_edit', __('Sorry, you are not allowed to edit this resource.', 'woocommerce'), array('status' => rest_authorization_required_code()));
    }

    return true;
  }
}
