<?php
if (!defined('ABSPATH')) {
  exit;
}

require_once 'class-customeow.php';

class CustoMeow_Template {
  public static function init() {
    $template = new self;
    $template->hook_templates();
  }

  public function hook_templates() {
    // add_action('woocommerce_after_add_to_cart_button', array($this, 'customeow_template_customize_button'), 20);
    // 按钮有可能呗别的插件删除该用 form 元素 hook
    add_action('woocommerce_after_add_to_cart_form', array($this, 'customeow_template_customize_button'), 20);


    // 额外数据显示
    add_filter('woocommerce_add_cart_item_data', array($this, 'add_woocommerce_cart_item_data'), 10, 1);
    add_filter('woocommerce_get_item_data', array($this, 'display_custom_data_in_woocommerce_cart'), 10, 2);
    add_action('woocommerce_checkout_create_order_line_item', array($this, 'add_custom_data_to_woocommerce_order_items'), 10, 4);

    // 隐藏以 _hidden_ 开头的订单元数据
    add_filter('woocommerce_hidden_order_itemmeta', array($this, 'hide_custom_order_item_meta'));
    
    // ajax 回调
    add_action('wp_ajax_add_to_woocommerce_cart_from_customeow', array($this, 'add_to_woocommerce_cart_from_customeow'));
    add_action('wp_ajax_nopriv_add_to_woocommerce_cart_from_customeow', array($this, 'add_to_woocommerce_cart_from_customeow')); // 处理未登录用户的 AJAX 请求
    add_action('wp_ajax_change_variation_from_customeow', array('CustoMeow', 'change_variation_from_customeow'));
    add_action('wp_ajax_nopriv_change_variation_from_customeow', array('CustoMeow', 'change_variation_from_customeow')); // 处理未登录用户的 AJAX 请求
    add_action('wp_ajax_abnormal_report_from_customeow', array('CustoMeow', 'abnormal_report_from_customeow'));
    add_action('wp_ajax_nopriv_abnormal_report_from_customeow', array('CustoMeow', 'abnormal_report_from_customeow')); // 处理未登录用户的 AJAX 请求
  }

  public static function customeow_template_customize_button() {
    global $post;

    CustoMeow_Admin::load_template('root', array('cpp_warp' => "customeow_pp_button_warp", 'cpp_root' => "customeow_pp_button_root"));

    $meta_data = get_post_meta($post->ID, 'customType', true);

    // 当前版本支持的定制类型
    $types = array(
      '1' => 'Customeow',
    );
    

    if (!empty($types[$meta_data]) && CustoMeow_Integration::instance()->is_connected()) {
      call_user_func(array($types[$meta_data], 'init'));
    }
  }

  /** 隐藏以 _hidden_ 开头的订单元数据 */
  public function hide_custom_order_item_meta($hidden_fields) {
    $hidden_fields[] = '_hidden_variant_title';
    $hidden_fields[] = '_hidden_custom_info_id';
    return $hidden_fields;
  }

  /** 添加加车数据到订单 */
  public static function add_custom_data_to_woocommerce_order_items($item, $cart_item_key, $values, $order) {
    if (isset($values['custom_info_id']) && !$values['is_upsell']) {
      // 添加额外的数据到订单（拉单需要）
      $item->add_meta_data('_hidden_' . 'variant_title', $values['variant_title']);
      $item->add_meta_data('_hidden_' . 'custom_info_id', $values['custom_info_id']);
    }

    if (empty($values['properties'])) {
      return;
    }
    foreach ($values['properties'] as $custom_data) {

      $custom_name = $custom_data['key']? $custom_data['propertyName'].'.'.$custom_data['key'] : $custom_data['propertyName'];
      $custom_value = is_array($custom_data['value']) ? implode(',', $custom_data['value']) : $custom_data['value'];

      $item->add_meta_data($custom_name, $custom_value);

    }
  }

  /** 添加加车数据到购物车 */
  public static function add_woocommerce_cart_item_data($cart_item_data) {
    $properties_value = [];
    if (isset($_POST['custom_info_id']) && !$cart_item_data['is_upsell']) { 
      // 添加额外的数据到订单（拉单需要）
      $cart_item_data['variant_title'] = $_POST['variant_title'];
      $cart_item_data['custom_info_id'] = $_POST['custom_info_id'];
    }
    // 如果有定制数据 并且不是追销产品 才需要添加到元数据
    if (isset($_POST['properties']) && !$cart_item_data['is_upsell']) {

      if (!isset($_POST['security'])) {
        check_ajax_referer('add_to_woocommerce_cart_from_customeow', 'security');
      }

      $properties_value['properties'] = $_POST['properties'];

      if (empty($cart_item_data)) {
        return $properties_value;
      } else {
        return array_merge($cart_item_data, $properties_value);
      }
    }
    return $cart_item_data;
  }

  /** 在购物车和结算中显示元数据 */
  public static function display_custom_data_in_woocommerce_cart($item_data, $cart_item) {

    if (!empty($cart_item['properties'])) {
      foreach ($cart_item['properties'] as $custom_data) {

        $custom_name = isset($custom_data['key']) ? $custom_data['propertyName'].'.'.$custom_data['key'] : $custom_data['propertyName'];
        $custom_value = is_array($custom_data['value']) ? implode(',', $custom_data['value']) : $custom_data['value'];

        $item_data[] = array(
          'name' => $custom_name,
          'value' => $custom_value,
          'display' => $custom_value,
        );
      }
    }
    return $item_data;
  }

  public static function add_to_woocommerce_cart_from_customeow() {
    check_ajax_referer('add_to_woocommerce_cart_from_customeow', 'security');

    // 检查并获取参数
    $product_id = isset($_POST['product_id']) ? intval($_POST['product_id']) : 0;
    $quantity = isset($_POST['quantity']) ? intval($_POST['quantity']) : 0;
    $variation_id = isset($_POST['variation_id']) ? intval($_POST['variation_id']) : 0;
    $upsells = isset($_POST['upsells']) ? $_POST['upsells'] : [];

    if ($product_id <= 0 || $quantity <= 0 || $variation_id <= 0) {
      wp_send_json_error(__('Required parameters are missing or invalid', 'customeow-product-personalizer'));
      wp_die();
    }

    $product = wc_get_product($product_id);
    $product_variation = wc_get_product($variation_id);
    $variation = $product_variation->get_attributes();
    $variant_options = self::convert_array_to_object_array($variation);
    $variant_options_with_values = self::convert_associative_to_indexed_array($variation);
    $variation_price = $product_variation->get_price();
    $variation_name = $product_variation->get_name();
    $variation_image = wp_get_attachment_image_url($product_variation->get_image_id(), 'medium');

    // 生成一个随机的 key , 用于同一产品分开显示在购物车
    $random_key = wp_generate_password(12, false);
    $cart_item_data['random_key'] = $random_key;
    $cart_item_key = WC()->cart->add_to_cart($product_id, $quantity, $variation_id, $variation, $cart_item_data);

    // 添加 upsell 到购物车
    foreach ($upsells as $upsell) {
      $upsell_variation = [];
      if (isset($upsell['variation_id']) && $upsell['variation_id'] > 0) {
        // 变体产品
        $upsell_variation = wc_get_product($upsell['variation_id'])->get_attributes();
      } 
      WC()->cart->add_to_cart($upsell['product_id'], $upsell['quantity'], $upsell['variation_id'], $upsell_variation, array('is_upsell' => true));
    }

    $response = array(
      'id' => $variation_id,
      'product_id' => $product_id,
      'title' => $variation_name,
      'variant_options' => $variant_options_with_values,
      'price' => $variation_price,
      'quantity' => $quantity,
      'image' => $variation_image,
      'options_with_values' => $variant_options,
      'variations' => $product->get_available_variations(),
    );

    if ($cart_item_key) {
      $response['cart_item_key'] = $cart_item_key;
      wc_add_to_cart_message($product_id, true); // 添加成功消息
      $notices = wc_print_notices(true) ;
      // 返回成功的响应
      wp_send_json_success(array('message' => __('Adding a cart successfully!', 'customeow-product-personalizer'), 'detail' => $response, 'notices' => $notices ));
    } else {
      wp_send_json_error(array('message' => __('Failed to add a cart!', 'customeow-product-personalizer')));
    }

    wp_die();
  }

  /**
   * 将变种关联数组转成数组对象 ['size'=> 'M'] => [{name:"size",value:"M"}]
   *
   */
  private static function convert_array_to_object_array($input_array) {
    $result = array();
    foreach ($input_array as $key => $value) {
      $result[] = array("name" => $key, "value" => $value);
    }
    return $result;
  }

  /**
   * 将变种关联数组转成字符串数组 ['size'=> 'M'] => ["M"]
   *
   */
  private static function convert_associative_to_indexed_array($input_array) {
    $result_array = array();
    foreach ($input_array as $value) {
      $result_array[] = $value;
    }
    return $result_array;
  }
}