<?php
if (!defined('ABSPATH')) {
  exit;
}

// 加载环境配置
require_once plugin_dir_path(__FILE__) . 'class-customeow-env-config.php';

if (!defined('CUSTOMEOW_ENV')) {
    define('CUSTOMEOW_ENV', 'dev'); // 默认环境
}

class CustoMeow_Utilities {
  const VERSION = '0.1.0';
  const ENV = CUSTOMEOW_ENV;

  /**
   * 获取 customeow umd 域名
   * @return {String}
   */
  public static function get_customeow_umd_host() {
    switch (self::ENV) {
    case 'dev':
    case 'beta':
    case 'uat':
      return 'https://assets.staticmeow.com/libumd/customeow.app.v1.37.8.js';
    case 'prod':
      return 'https://assets.staticmeow.com/libumd/customeow.app.v1.37.8.js';
    default:
      return '';
    }
  }

  /**
   * 获取 customeow json 域名
   * @return {String}
   */
  public static function get_customeow_json_host() {
    switch (self::ENV) {
    case 'dev':
    case 'beta':
      return 'https://sunzi-cn.oss-cn-hangzhou.aliyuncs.com/';
    case 'uat':
      return 'https://sunzi-us-east.oss-us-east-1.aliyuncs.com/';
    case 'prod':
      return 'https://json.staticmeow.com/';
    default:
      return '';
    }
  }

  /**
   * 获取 customeow go 域名
   * @return {String}
   */
  public static function get_customeow_go_host() {
    switch (self::ENV) {
    case 'dev':
      return 'https://api-go-dev-customeow.maiyuan.online/';
    case 'beta':
      return 'https://api-go-beta-customeow.maiyuan.online/';
    case 'uat':
      return 'https://api-go-uat-customeow.maiyuan.online/';
    case 'prod':
      return 'https://api-go.customeow.io/';
    default:
      return '';
    }
  }

    /**
   * 获取 定制喵 域名
   * @return {String}
   */
  public static function get_customeow_host() {
    switch (self::ENV) {
    case 'dev':
      return 'https://dev-customeow.maiyuan.online/';
    case 'beta':
      return 'https://beta-customeow.maiyuan.online/';
    case 'uat':
      return 'https://uat-customeow.maiyuan.online/';
    case 'prod':
      return 'https://customeow.io/';
    default:
      return '';
    }
  }

  /**
   * 获取当前货币
   * @return {String}
   */
  public static function get_current_currency() {
    // 获取当前货币
    $currency = get_woocommerce_currency();
    // 定制喵币种列表 
    $customeow_currency_map = ['USD', 'EUR', 'GBP', 'CAD', 'ARS', 'AUD', 'BBD', 'BDT', 'BSD', 'BHD', 'BRL', 'BOB', 'BND', 'BGN', 'MMK', 'KYD', 'CLP', 'CNY', 'COP', 'CRC', 'HRK', 'CZK', 'DKK', 'DOP', 'XCD', 'EGP', 'XPF', 'FJD', 'GHS', 'GTQ', 'GYD', 'GEL', 'HKD', 'HUF', 'ISK', 'INR', 'IDR', 'NIS', 'JMD', 'JPY', 'JOD', 'KZT', 'KES', 'KWD', 'LVL', 'LTL', 'MXN', 'MYR', 'MUR', 'MDL', 'MAD', 'MNT', 'MZN', 'ANG', 'NZD', 'NGN', 'NOK', 'OMR', 'PKR', 'PYG', 'PEN', 'PHP', 'PLN', 'QAR', 'RON', 'RUB', 'SAR', 'RSD', 'SCR', 'SGD', 'SYP', 'ZAR', 'KRW', 'LKR', 'SEK', 'CHF', 'TWD', 'THB', 'TZS', 'TTD', 'TRY', 'UAH', 'AED', 'UYU', 'VEB', 'VND', 'ZMK', 'XBT'];

    // 转化大写
    $currency = strtoupper($currency);
    // 不存在则使用美元币种
    return in_array($currency, $customeow_currency_map) ? $currency : 'USD';
  }


  /**
   * 获取当前语言
   * @return {String}
   */
  public static function get_current_language() {
    $language = get_locale();

      /** 定制喵语种映射对象 */   
    $customeowLanguagesMap = [
      'zh_CN' => 'zh_CN',
      'zh_TW' => 'zh_TW',
      'pt_BR' => 'pt_BR',
      'pt_PT' => 'pt_PT',
      'en_US' => 'en_US',
      'fr_FR' => 'fr_FR',
      'es_ES' => 'es_ES',
      'de_DE' => 'de_DE',
      'it_IT' => 'it_IT',
      'ru_RU' => 'ru_RU',
      'ar' => 'ar_AR',
      'th' => 'th_TH',
      'id_ID' => 'id_ID',
      'nl_NL' => 'nl_NL',
      'tr_TR' => 'tr_TR',
      'vi' => 'vi_VI',
      'he_IL' => 'he_IL',
      'sv_SE' => 'sv_SV',
      'pl_PL' => 'pl_PL',
      'ko_KR' => 'ko_KR',
      'ja' => 'ja_JP',
      'el' => 'el_GR',
      'cs_CZ' => 'cs_CZ',
      'ro_RO' => 'ro_RO',
      'sk_SK' => 'sk_SK',
      'hu_HU' => 'hu_HU',
      'bg_BG' => 'bg_BG',
  ];

    return $customeowLanguagesMap[$language] ?? 'en_US';
  }

}
