<?php
if (!defined('ABSPATH')) {
  exit;
}

class CustoMeow {
  private $VERSION;

  public function __construct() {
    $this->VERSION = CustoMeow_Utilities::VERSION;
  }

  public static function init() {
    $template = new self;

    $template->add_customeow_umd_scripts();
    $template->add_customeow_scripts();
  }

  /**
   * @description: 加载 customeow umd 文件
   * @return {*}
   */
  public function add_customeow_umd_scripts() {
    wp_register_script(
      'customeow_umd',
      CustoMeow_Utilities::get_customeow_umd_host(),
      array(),
      $this->VERSION,
      true
    );
    wp_enqueue_script('customeow_umd', null, array('jquery'), $this->VERSION, true);
  }

  /**
   * @description: 加载前端 js
   * @return {*}
   */
  public function add_customeow_scripts() {
    $script_path = '/build/designer-bootstrap.js';
    $script_asset_path = dirname(CUSTOMEOW_PP_PLUGIN_FILE) . '/build/designer-bootstrap.asset.php';
    $script_asset = file_exists($script_asset_path)
    ? require $script_asset_path
    : array(
      'dependencies' => array(),
      'version' => filemtime($script_path),
    );
    $script_url = plugins_url($script_path, CUSTOMEOW_PP_PLUGIN_FILE);

    wp_register_script(
      "customeow_js",
      $script_url,
      array_merge(array('jquery'), $script_asset['dependencies']),
      $script_asset['version'],
      true
    );
    wp_enqueue_script('customeow_js', null, array('customeow_umd'), $this->VERSION, true);

    wp_register_style(
      'customeow_style',
      plugins_url('/build/designer-bootstrap.css', CUSTOMEOW_PP_PLUGIN_FILE),
      array(),
      filemtime(dirname(CUSTOMEOW_PP_PLUGIN_FILE) . '/build/designer-bootstrap.css')
    );
    wp_enqueue_style('customeow_style');

    CustoMeow::render_connect();
  }

  public static function render_connect() {
    global $post;
    $product = wc_get_product($post->ID);

    $product_id = $product->get_id();
    // 非变体产品不支持
    if (!$product->is_type('variable')) {
      return;
    }
    //     // 获取产品类型相关数据
    // if ($product->is_type('variable')) {
    //     // ... 现有的变体产品处理代码 ...
    // } else {
    //     // 处理简单产品
    //     $default_variation_id = $product_id;
    //     $variant_sku = $product->get_sku();
        
    //     if (empty($variant_sku)) {
    //         return;
    //     }
        
    //     $image_id = $product->get_image_id();
    //     $image_url = wp_get_attachment_image_url($image_id, 'full');
    //     $store_id = CustoMeow_Integration::instance()->get_option('store_id');
    //     $platform_id = CustoMeow_Integration::instance()->get_option('platform_id');

    //     $added_response = array(
    //         'variantId' => $default_variation_id,
    //         'featuredImage' => $image_url,
    //         'available' => $product->is_in_stock(),
    //         'platformId' => $platform_id,
    //         'sku' => $variant_sku
    //     );
        
    //     // 后续处理与变体产品相同
    //     // ... 其余代码保持不变 ...
    // }
    $default_attributes = $product->get_default_attributes();
    $available_variations = $product->get_available_variations();

    // 根据变体默认属性查找变体 ID
    $default_variation_id = 0;
    $variant_sku = '';
    foreach ($available_variations as $variation) {
      $match = true;
      foreach ($default_attributes as $attribute_name => $attribute_value) {
        if ($variation['attributes']['attribute_' . $attribute_name] !== $attribute_value) {
          $match = false;
          break;
        }
      }
      if ($match) {
        $default_variation_id = $variation['variation_id'];
        $variant_sku = $variation['sku'];
        break;
      }
    }

    if (empty($default_variation_id)) {
      return;
    }

    // 获取默认变体的产品对象
    $variation_product = wc_get_product($default_variation_id);

    $image_id = $variation_product->get_image_id();
    $image_url = wp_get_attachment_image_url($image_id, 'full');
    $store_id = CustoMeow_Integration::instance()->get_option('store_id');
    $platform_id = CustoMeow_Integration::instance()->get_option('platform_id');


    $added_response = array(
      'variantId' => $default_variation_id,
      'featuredImage' => $image_url,
      'available' => $variation_product->is_in_stock(),
      'platformId' => $platform_id,
      'sku' => $variant_sku
    );

    // 获取 json 定制数据
    $customeow_url = CustoMeow_Utilities::get_customeow_json_host();
    $client = CustoMeow_Integration::instance()->get_client($customeow_url);
    $current_env = CustoMeow_Utilities::ENV;
    $cm_namespace = $current_env === 'prod' ? 'customeow' : 'customeow-' . $current_env;
    $url = '/'. $cm_namespace .'/app/woocommerce/'. $store_id. '/' . $variant_sku . '.json';
    // $url = '/'. $cm_namespace .'/app/woocommerce/'. '150602849078747136'. '/' . 'asd99f5df' . '.json';

    try {
      $response = $client->get($url) ?? [];
      // if (empty($response)) {
      //   wp_localize_script('customeow_js', 'customeow_api_response', array(
      //     "url" => $customeow_url . trim($url, '/'),
      //   ));
      //   return;
      // }

      $response = array_merge($added_response, $response);

      $shop_config = CustoMeow_Integration::instance()->get_option("shop_config");

      if (!$shop_config) {
        $shop_config = null;
      } else {
        // 注入货币
        $shop_config['currency'] = CustoMeow_Utilities::get_current_currency();
        // 注入语种
        $shop_config['language'] = CustoMeow_Utilities::get_current_language();
        // 调试原始语种
        // $shop_config['origin_language'] = get_locale();
      }

      wp_localize_script('customeow_js', 'customeow_front_end_data', array(
        'ajax_url' => esc_url(admin_url('admin-ajax.php')),
        'cart_url' => esc_url(wc_get_cart_url()),
        'customeow_json_url' => $url,
        'customeow_umd_url' => CustoMeow_Utilities::get_customeow_umd_host(),
        "security" => wp_create_nonce('add_to_woocommerce_cart_from_customeow'),
        'shop_config' => $shop_config,
        'response' => $response,
        'custom_type' => get_post_meta($post->ID, 'customType', true),
        'variation_id' => $default_variation_id,
        'product_id' => $product_id,
      ));

    } catch (Exception $e) {
      return false;
    }
  }

  /**
   * 根据不同的变体产品请求不同的JSON数据
   *
   */
  public static function change_variation_from_customeow() {

    check_ajax_referer('add_to_woocommerce_cart_from_customeow', 'security');

    $variation_id = isset($_POST['variation_id']) ? intval($_POST['variation_id']) : 0;
    $product_id = isset($_POST['product_id']) ? intval($_POST['product_id']) : 0;

    if ($variation_id <= 0) {
      wp_send_json_error(__('Required parameters are missing or invalid', 'customeow-product-personalizer'));
      wp_die();
    }

    // 获取默认变体的产品对象
    $variation_product = wc_get_product($variation_id);
    $product = wc_get_product($product_id);
    $available_variations = $product->get_available_variations();

    // 根据变体默认属性查找变体 ID
    $variant_sku = '';
    foreach ($available_variations as $variation) {
      if ($variation['variation_id'] == $variation_id) {
        $variant_sku = $variation['sku'];
        break;
      }
    }

    $image_id = $variation_product->get_image_id();
    $image_url = wp_get_attachment_image_url($image_id, 'full');
    $store_id = CustoMeow_Integration::instance()->get_option('store_id');
    $platform_id = CustoMeow_Integration::instance()->get_option('platform_id');

    $added_response = array(
      'variantId' => $variation_id,
      'featuredImage' => $image_url,
      'available' => $variation_product->is_in_stock(),
      'sku' => $variant_sku,
      'platformId' => $platform_id
    );

    // 获取 json 定制数据
    $customeow_url = CustoMeow_Utilities::get_customeow_json_host();
    $client = CustoMeow_Integration::instance()->get_client($customeow_url);
    $current_env = CustoMeow_Utilities::ENV;
    $cm_namespace = $current_env === 'prod' ? 'customeow' : 'customeow-' . $current_env;
    $url = '/'. $cm_namespace .'/app/woocommerce/'. $store_id. '/' . $variant_sku . '.json';

    try {
      $response = $client->get($url);

      if (empty($response)) {
        wp_localize_script('customeow_js', 'customeow_api_response', array(
          "url" => $customeow_url . trim($url, '/'),
        ));
        wp_send_json(array(
          'success' => false,
        ));
        return;
      }

      $response = array_merge($added_response, $response);

      wp_send_json(array(
        'success' => true,
        'data' => $response,
      ));

    } catch (Exception $e) {
      wp_send_json(array(
        'success' => false,
      ));
    }

    wp_die();
  }

  /**
   * 上报错误
   */
  public static function abnormal_report_from_customeow() {
    check_ajax_referer('add_to_woocommerce_cart_from_customeow', 'security');

    $platform_id = isset($_POST['platform_id']) ? intval($_POST['platform_id']) : 0;

    if ($platform_id <= 0) {
      wp_send_json_error(__('Required parameters are missing or invalid', 'customeow-product-personalizer'));
      wp_die();
    }

    // 上报错误
    $customeow_url = CustoMeow_Utilities::get_customeow_go_host();
    $client = CustoMeow_Integration::instance()->get_client($customeow_url);
    $url = '/api/v1/store/user-stores-err-operation';

    try {
      $response = $client->post($url, array(
        'platformId' => $platform_id,
      ));

      if (empty($response)) {
        wp_localize_script('customeow_js', 'customeow_api_response', array(
          "url" => $customeow_url . trim($url, '/'),
        ));
        wp_send_json(array(
          'success' => false,
        ));
        return;
      }


      wp_send_json(array(
        'success' => true,
        'data' => $response,
      ));

    } catch (Exception $e) {
      wp_send_json(array(
        'success' => false,
      ));
    }

    wp_die();
  }
}
