/* eslint-disable @typescript-eslint/no-use-before-define */
/* eslint-disable @wordpress/i18n-text-domain */
/**
 * External dependencies
 */
import { sprintf, __ } from '@wordpress/i18n';
import {
	Fragment,
	useState,
	createRoot,
	useRef,
	useEffect,
} from '@wordpress/element';
import domReady from '@wordpress/dom-ready';

/**
 * Internal dependencies
 */
import CheckSvg from './components/checkSvg';
import Arrow from './components/arrow';
import Copy from './components/copy';
import Message from './components/message';
import Error from './components/error';
import './index.scss';
import guide from '../../assets/guide.png';

const CustoMeowAdminPage = () => {
	const { customeow_admin_data = undefined } = window;

	const [ showMessage, setShowMessage ] = useState( { value: false } );
	const [ isConnect, setIsConnect ] = useState(
		customeow_admin_data?.connect_status === '0' ? false : true
	);
	const [ storeUrl, setStoreUrl ] = useState(
		customeow_admin_data?.store_url ?? 'https://customeow.io'
	);
	const [ isPolling, setIsPolling ] = useState( false );
	const pollingRef = useRef( null );
	const startTimeRef = useRef( null );

	useEffect( () => {
		if ( ! customeow_admin_data ) return;

		if ( customeow_admin_data.connect_status === '0' && ! isPolling ) {
			setIsPolling( true );
			startTimeRef.current = new Date().getTime();
			poll();
		}

		return () => {
			clearTimeout( pollingRef.current );
		};

		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, [] );

	const poll = () => {
		if ( ! customeow_admin_data ) return;

		const { ajax_url, security } = customeow_admin_data;

		window.jQuery.ajax( {
			url: ajax_url,
			method: 'POST',
			data: {
				action: 'ajax_force_check_customeow_connect_status',
				security,
			},
			success: ( response ) => {
				if ( response.data.is_connect ) {
					clearTimeout( pollingRef.current );
					setIsConnect( true );
					setStoreUrl( response.data.store_url );
					setIsPolling( false );
				} else {
					checkPollingTime();
				}
			},
			error: () => {
				checkPollingTime();
			},
		} );
	};

	const checkPollingTime = () => {
		const currentTime = new Date().getTime();
		if ( currentTime - startTimeRef.current >= 10 * 60 * 1000 ) {
			clearTimeout( pollingRef.current );
			setIsPolling( false );
		} else {
			pollingRef.current = setTimeout( poll, 10000 );
		}
	};

	const unsecuredCopyToClipboard = ( text ) => {
		const textArea = document.createElement( 'textarea' );
		textArea.style.position = 'fixed';
		textArea.style.top = 0;
		textArea.style.left = 0;
		textArea.style.opacity = 0;

		textArea.value = text;
		document.body.appendChild( textArea );
		textArea.focus();
		textArea.select();
		try {
			document.execCommand( 'copy' );
			setShowMessage( { value: true } );
		} catch ( err ) {
			const msg = __(
				'Unable to copy to clipboard',
				'customeow-product-personalizer'
			);
			console.error( msg, err );
		}
		document.body.removeChild( textArea );
	};

	const handleCopy = async () => {
		if ( window.isSecureContext && window.navigator.clipboard ) {
			window.navigator.clipboard.writeText(
				customeow_admin_data?.home_url
			);
			setShowMessage( { value: true } );
		} else {
			unsecuredCopyToClipboard( customeow_admin_data?.home_url );
		}
	};

	const handlerToCustoMeow = () => {
		window.open( storeUrl );
	};

	const handlerConnect = () => {
		if ( customeow_admin_data ) {
			const connect_url = customeow_admin_data.connect_url;
			window.open( connect_url );
		}
	};

	// if ( customeow_admin_data && customeow_admin_data.issues.length !== 0 ) {
	// 	return (
	// 		<>
	// 			<Error errors={ customeow_admin_data?.issues ?? [] } />
	// 		</>
	// 	);
	// }

	return (
		<Fragment>
			<Message
				show={ showMessage }
				message={ __(
					'🎉 Copied successfully！！！',
					'customeow-product-personalizer'
				) }
			/>

			<div className="cm-contanier">
				<div className="cm-contanier-item">
					<div className="cm-contanier-item-check">
						<CheckSvg />
					</div>
					<div className="cm-contanier-item-right">
						<div className="cm-contanier-item-right-title">
							{ __(
								'Install CustoMeow plugin',
								'customeow-product-personalizer'
							) }
						</div>
						<div className="cm-contanier-item-right-sub-title">
							{ __(
								'CustoMeow plugin installation was successful! 🎉',
								'customeow-product-personalizer'
							) }
						</div>
					</div>
				</div>
				<div className="cm-contanier-item">
					{ isConnect ? (
						<div className="cm-contanier-item-check">
							<CheckSvg />
						</div>
					) : (
						<div className="cm-contanier-item-left"></div>
					) }
					<div className="cm-contanier-item-right">
						<div className="cm-contanier-item-right-title">
							{ __(
								'Connect WooCommerce store to CustoMeow',
								'customeow-product-personalizer'
							) }
						</div>
						<div className="cm-contanier-item-right-sub-title">
							{ isConnect
								? sprintf(
										// translators: The following string contains a placeholder. Replace %s with the current site name.
										__(
											'%s has been successfully connected to CustoMeow! 🎉',
											'customeow-product-personalizer'
										),
										customeow_admin_data?.name
								  )
								: sprintf(
										// translators: The following string contains a placeholder. Replace %s with the current site name.
										__(
											'You only need to register or login to CustoMeow, connect %s to CustoMeow',
											'customeow-product-personalizer'
										),
										customeow_admin_data?.name
								  ) }
						</div>
						<div className="cm-contanier-item-right-connect">
							{ isConnect ? (
								// eslint-disable-next-line jsx-a11y/click-events-have-key-events, jsx-a11y/no-static-element-interactions
								<div
									className="cm-contanier-item-right-connect-button"
									onClick={ handlerToCustoMeow }
								>
									<span className="cm-contanier-item-right-connect-button-name">
										{ __(
											'Go to CustoMeow',
											'customeow-product-personalizer'
										) }
									</span>
								</div>
							) : (
								// eslint-disable-next-line jsx-a11y/click-events-have-key-events, jsx-a11y/no-static-element-interactions
								<div
									className="cm-contanier-item-right-connect-button"
									onClick={ handlerConnect }
								>
									<span className="cm-contanier-item-right-connect-button-name">
										{ __(
											'Go to connect ',
											'customeow-product-personalizer'
										) }
									</span>
									<span className="cm-contanier-item-right-connect-button-arrow">
										<Arrow />
									</span>
								</div>
							) }
						</div>
						{ isConnect ? null : (
							<>
								{ /* eslint-disable-next-line jsx-a11y/click-events-have-key-events, jsx-a11y/no-static-element-interactions */ }
								<div
									className="cm-contanier-item-right-copy-site"
									onClick={ handleCopy }
								>
									<span className="cm-contanier-item-right-copy-site-name">
										{ customeow_admin_data?.home_url }
									</span>
									<span className="cm-contanier-item-right-copy-site-icon">
										<Copy></Copy>
									</span>
								</div>
								<div className="cm-contanier-item-right-img">
									<img
										src={ guide }
										alt=""
										className="cm-contanier-item-right-img-guide"
									/>
								</div>
							</>
						) }
					</div>
				</div>
			</div>
		</Fragment>
	);
};

domReady( () => {
	const customeowText =
		'' +
		'     ____                   __                                                 ' +
		'\n' +
		"    /\\  _`\\                /\\ \\__         /'\\_/`\\                              " +
		'\n' +
		'    \\ \\ \\/\\_\\  __  __   ___\\ \\ ,_\\   ___ /\\      \\     __    ___   __  __  __  ' +
		'\n' +
		"     \\ \\ \\/_/_/\\ \\/\\ \\ /',__\\ \\ \\/  / __`\\ \\ \\__\\ \\  /'__`\\ / __`\\/\\ \\/\\ \\/\\ \\ " +
		'\n' +
		'      \\ \\ \\L\\ \\ \\ \\_\\ /\\__, `\\ \\ \\_/\\ \\L\\ \\ \\ \\_/\\ \\/\\  __//\\ \\L\\ \\ \\ \\_/ \\_/ \\ ' +
		'\n' +
		"       \\ \\____/\\ \\____\\/\\____/\\ \\__\\ \\____/\\ \\_\\\\ \\_\\ \\____\\ \\____/\\ \\___x___/' " +
		'\n' +
		'        \\/___/  \\/___/ \\/___/  \\/__/\\/___/  \\/_/ \\/_/\\/____/\\/___/  \\/__//__/  ' +
		'\n' +
		'';

	console.log(
		customeowText +
			'\n\n' +
			'        Build custom products better.\n\n' +
			'        developer@customeow.io\n\n'
	);

	const rootElement = document.getElementById( 'customeow_pp_admin_root' );
	if ( rootElement ) {
		const root = createRoot( rootElement );

		root.render( <CustoMeowAdminPage /> );
	}
} );
